{% extends "admin/change_list.html" %}

{% block content %}
<noscript>
    <a href="{% url treenav_refresh_hrefs %}"
        title="Refresh all cached menu item HREFs in the database">
        Refresh HREFs
    </a>
</noscript>
{{ block.super }}
<script type="text/javascript">
(function($) {
  $('.object-tools').prepend('<li><a href="{% url treenav_refresh_hrefs %}" title="Refresh all cached menu item HREFs in the database">Refresh HREFs</a></li>');
  $('.object-tools').prepend('<li><a href="{% url treenav_clean_cache %}" title="Remove all MenuItems from Cache.">Clean Menu Cache</a></li>');
})(django.jQuery);
</script>
{% endblock %}
